/* Foldroid - Mail Folder Selection Tool
 * Copyright (C) 2000-2010 Stefan Westerfeld
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include "fddconfig.h"
#include "fddpattern.h"
#include "microconf.hh"
#include <fstream>
#include <assert.h>
#include <stdlib.h>
#include "boost/filesystem.hpp"

using namespace boost::filesystem;
using namespace std;
using namespace Foldroid;

static bool
is_pattern (const string& pattern)
{
  for (string::const_iterator pi = pattern.begin(); pi != pattern.end(); pi++)
    {
      if (*pi == '*')
        return true;
    }
  return false;
}

void
Config::add_folders (const string& str, const string& label = "")
{
  string filename = str;

  if (is_pattern (filename))
    {
      string folder_dir;
      bool relative_path = (filename[0] != '/');
      if (!relative_path)    // absolute path
        {
          folder_dir = path (filename).branch_path().string();
          filename = basename (filename);
        }
      else                  // relative path
        {
          folder_dir = m_folder_dir;
        }
      directory_iterator end_itr; // default construction yields past-the-end
      for (directory_iterator itr (folder_dir); itr != end_itr; ++itr)
        {
          if (pattern_match (filename, itr->leaf()))
            {
              FolderConfig folder_config;
              folder_config.filename = m_folder_dir + "/" + itr->leaf();
              folder_config.label = relative_path ? itr->leaf() : folder_config.filename;

              if (label != "")
                folder_config.label = label;

              m_folders.push_back (folder_config);
            }
        }
    }
  else
    {
      FolderConfig folder_config;

      folder_config.label = filename;
      if (str[0] == '/')    // absolute path
        folder_config.filename = str;
      else
        folder_config.filename = m_folder_dir + "/" + filename;

      if (label != "")
        folder_config.label = label;

      m_folders.push_back (folder_config);
    }
}

Config::Config (const string& filename) : m_filename (filename)
{
  m_open_command = "screen mutt -f '%s'";   // default

  char *home = getenv ("HOME");
  if (home)
    m_folder_dir = home + string ("/Mail");
  else
    m_folder_dir = ".";

  MicroConf cfg (m_filename);

  if (!cfg.open_ok())
    {
      fprintf (stderr, "missing configuration file: %s\n", m_filename.c_str());
      exit (1);
    }

  while (cfg.next())
    {
      string str, label;
      if (cfg.command ("folder_dir", str))
        {
          m_folder_dir = str;
        }
      else if (cfg.command ("folder", str, label))
        {
          add_folders (str, label);
        }
      else if (cfg.command ("folder", str))
        {
          add_folders (str);
        }
      else if (cfg.command ("open_command", str))
        {
          m_open_command = str;
        }
      else
        {
          cfg.die_if_unknown();
        }
    }
}

string
Config::open_command()
{
  return m_open_command;
}

vector<FolderConfig>
Config::folders()
{
  return m_folders;
}

/* vim:set ts=8 sw=2 sts=2: */
