/* Foldroid - Mail Folder Selection Tool
 * Copyright (C) 2000-2010 Stefan Westerfeld
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place, Suite 330,
 * Boston, MA 02111-1307, USA.
 */

#include <curses.h>
#include <menu.h>
#include <malloc.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include "fddconfig.h"
#include "fddscan.h"
#include "foldroid.h"
#include "fddfolderthread.h"
#include "config.h"

using std::string;
using std::vector;
using std::list;

#define CTRLD   4


using namespace Foldroid;

int
main()
{
  FolderThread::start();
  UserInterface ui;

  int result = ui.main();
  FolderThread::stop();
}

UserInterface::UserInterface() :
  m_config (default_config())
{
}

void
UserInterface::init_curses()
{
  /* Initialize curses */
  initscr();
  start_color();
  halfdelay(10);
  noecho();
  keypad(stdscr, TRUE);
  curs_set (0);
  refresh ();
  init_pair(1, COLOR_RED, COLOR_BLACK);
  init_pair(2, COLOR_CYAN, COLOR_BLACK);
}

void
UserInterface::build_menu (const string& filename)
{
  if (m_scan.need_update())
    {
      print_status ("updating folders ...");
      m_scan.update (*this);
    }

  const list<Folder>& folders = m_scan.folders();
  m_n_choices = folders.size();

  ITEM *sel_item = NULL;
  int i = 0;
  /* Create items */
  m_my_items = (ITEM **)calloc (m_n_choices + 1, sizeof (ITEM *));
  m_item_filename.clear();
  for (list<Folder>::const_iterator fi = folders.begin(); fi != folders.end(); fi++)
    {
      char *name = strdup (fi->label().c_str());
      char *desc = strdup (fi->description().c_str());
      m_item_filename.push_back (fi->filename());

      m_my_items[i++] = new_item (name, desc);
      if (fi->filename() == filename)
        sel_item = m_my_items[i-1];
    }

  const int menu_width = COLS;
  int menu_height = LINES - 3;

  /* Create menu */
  m_my_menu = new_menu ((ITEM **)m_my_items);

  /* Create the window to be associated with the menu */
  m_my_menu_win = newwin (menu_height, menu_width, 0, 0);
  keypad (m_my_menu_win, TRUE);

  /* Set main window and sub window */
  set_menu_win (m_my_menu, m_my_menu_win);
  set_menu_sub (m_my_menu, derwin (m_my_menu_win, menu_height - 4, menu_width - 2, 3, 1));
  set_menu_format (m_my_menu, menu_height - 4, 1);

  /* Set menu mark to the string " * " */
  set_menu_mark (m_my_menu, " * ");

  /* Print a border around the main window and print a title */
  box (m_my_menu_win, 0, 0);
  print_in_middle (m_my_menu_win, 1, 0, menu_width, PACKAGE_NAME "-" PACKAGE_VERSION " - Mail Folder Selection Tool",
                   COLOR_PAIR(1));
  mvwaddch (m_my_menu_win, 2, 0, ACS_LTEE);
  mvwhline (m_my_menu_win, 2, 1, ACS_HLINE, menu_width - 2);
  mvwaddch (m_my_menu_win, 2, menu_width - 1, ACS_RTEE);

  if (sel_item)
    set_current_item (m_my_menu, sel_item);

  /* Post the menu */
  post_menu (m_my_menu);
  wrefresh (m_my_menu_win);

  print_status ("");
}

string
UserInterface::free_menu()
{
  string selected_filename = m_item_filename[item_index (current_item (m_my_menu))];

  wrefresh (m_my_menu_win);

  /* Unpost and free all the memory taken up */
  unpost_menu (m_my_menu);
  ::free_menu (m_my_menu);
  for (int i = 0; i < m_n_choices; ++i)
    free_item (m_my_items[i]);
  /* TODO: free strings */

  return selected_filename;
}

void
UserInterface::print_status (const char *status)
{
  string sstr = status;
  while (sstr.size() < 70)
    sstr = " " + sstr + " ";

  print_in_middle (stdscr, LINES - 3, 0, 0, sstr.c_str(), COLOR_PAIR(2));
}

int
UserInterface::main()
{
  init_curses();
  build_menu ("");

  print_in_middle (stdscr, LINES - 2, 0, 0, "Use PageUp and PageDown to scoll down or up a page of items", COLOR_PAIR(2));
  print_in_middle (stdscr, LINES - 1, 0, 0, "Arrow Keys to navigate (q to quit)", COLOR_PAIR(2));
  refresh();

  int c;
  int changes = 0;
  while ((c = wgetch (m_my_menu_win)) != 'q')
    {
      if (m_scan.need_update() || m_scan.changes() != changes)
        {
          changes = m_scan.changes();
          string selected_filename = free_menu();
          build_menu (selected_filename);
        }

      switch(c)
        {
          case KEY_DOWN:
            menu_driver (m_my_menu, REQ_DOWN_ITEM);
            break;
          case KEY_UP:
            menu_driver (m_my_menu, REQ_UP_ITEM);
            break;
          case KEY_NPAGE:
            menu_driver (m_my_menu, REQ_SCR_DPAGE);
            break;
          case KEY_PPAGE:
            menu_driver (m_my_menu, REQ_SCR_UPAGE);
            break;
          case 10: /* Enter */
            {
              const char *name = m_item_filename[item_index (current_item (m_my_menu))].c_str();
              string cmd = m_config.open_command();
              char buffer[1024];
              sprintf (buffer, cmd.c_str(), name);
              curs_set (1);
              reset_shell_mode();
              system (buffer);
              reset_prog_mode();
              clearok (stdscr, TRUE);
              curs_set (0);
              refresh();
            }
            break;
          case ' ':
            {
              string selected_filename = free_menu();
              build_menu (selected_filename);
            }
            break;
        }
    }

  free_menu();

  endwin();
}

void
UserInterface::print_in_middle(WINDOW *win, int starty, int startx, int width, const char *string, chtype color)
{
  int length, x, y;
  float temp;

  if (win == NULL)
    win = stdscr;
  getyx (win, y, x);
  if (startx != 0)
    x = startx;
  if (starty != 0)
    y = starty;
  if (width == 0)
    width = COLS;

  length = strlen(string);
  temp = (width - length)/ 2;
  x = startx + (int)temp;
  wattron(win, color);
  mvwprintw(win, y, x, "%s", string);
  wattroff(win, color);
  refresh();
}

string
UserInterface::default_config()
{
  char *home = getenv ("HOME");

  return home + string ("/.foldroidrc");
}
/* vim:set ts=8 sw=2 sts=2: */
